% Demo for Image Fusion Algorithm based on Bandelet Transform
% ---------
% Author:  Qu Xiao-Bo    <qxb_xmu [at] yahoo.com.cn>    Aug.28,2008
%          Postal address:
% Rom 509, Scientific Research Building # 2,Haiyun Campus, Xiamen University,Xiamen,Fujian, P. R. China, 361005
% Website: http://quxiaobo.8866.org or http://quxiaobo.go.8866.org
%=============================================================
% References:
% [1] Xiaobo Qu, Jingwen Yan, Guofu Xie, et al. A Novel Image Fusion Algorithm Based on Bandelet Transform. Chinese Optics Letters,Vol.5,No.10,pp:569-572, 2007
% (Check http://quxiaobo.8866.org, http://quxiaobo.go.8866.org, http://quxiaobo.blog.edu.cn or http://naec.stu.edu.cn for these and 
% other related papers.)
%=============================================================
%%
clc;
clear all;
close all;
%%
path(path,'bandelet_toolbox/')
path(path,'bandelet_toolbox/cwpt2/')
path(path,'FusionEvaluation/')
%% read the source image
nameA= 'hoed_A';
nameB= 'hoed_B';
%%
Geometry_Rule='average';% 'average' or 'maximum'
% 'average': compute the average geometry of the two source images
% 'maximum': compute the maximum geometry of the two source images
Low_Coeffs_Rule='maximum';% 'average' or 'maximum'
% 'average': compute the average of low-frequency bandelet coefficients of the two source images
% 'maximum': compute the maximum of low-frequency bandelet coefficients of the two source images
%%
MA= load_image(nameA);
MB= load_image(nameB);
Temp_M=(MA+MB)/2;
%%
[n,n]=size(MA);% get the size of image (require square image)
%%
M0A=MA;
M0B=MB;
%% TI Wavelets
clear options;
%%%%%%%%
sigma = 0.04 * max( M0A(:) );
T = 3*sigma;
%%%%%%%%
options.wavelet_type = 'biorthogonal';
options.wavelet_vm = 3;
Jmin = 4;
options.decomp_type = 'quad';

nbr_scales = 3; % number of scale on witch geometry is computed
bandlet_type = 'atrou-quad';
switch bandlet_type
    case 'atrou-quad'
        callback = @callback_atrou;
        options.decomp_type = 'quad';
        Jband = 3*nbr_scales;
    case 'atrou-tri'
        callback = @callback_atrou;
        options.decomp_type = 'tri';
        Jband = 2*nbr_scales;
    case 'steerable'
        options.nb_orientations = 1;
        callback = @callback_atrou;
        Jband = options.nb_orientations*nbr_scales+1;
    case 'laplacian'
        callback = @callback_laplacian;
        Jband = nbr_scales;
    otherwise
        error('Unknown type.');
end  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TI Bandlets
options.w = 8;
options.q = 8;  % for overlap set q=w/2, otherwise q=w
options.T = T;  % 10/256;
options.Jband = Jband; % number of scales for bandlets
options.Jhaar = 2; % number of scales for the 1D haar transform
options.callback = callback;
options.geometry_weight = 10/256; % price to pay for the geometry
options.n = n;
% use or not TI haar
options.ti = 1;
%%
% compute the geometry for the bandlet transform
disp('--> Computing geometry.');
switch Geometry_Rule
    case 'average'
%         G_ave= compute_geometry( Temp_M, options );
%         save G_ave G_ave
        load G_ave
        figure,plot_geometry(G_ave,Temp_M,options);title('Geometry of average of images A and B')
        GF=G_ave;
        figure,plot_geometry(G_ave,Temp_M,options);title('Geometry of average of fused image F')
    case 'maximum'
%         GA = compute_geometry( M0A, options );
%         GB = compute_geometry( M0B, options );
%         save GA GA
%         save GB GB
        load GA
        load GB
        figure,plot_geometry(GA,M0A,options),title('Geometry of image A')
        figure,plot_geometry(GB,M0B,options),title('Geometry of image B')
        for l = 1:length(GA)
            A_G=GA{l};
            B_G=GB{l};
            judge_G=abs(A_G)>=abs(B_G);
            GF{l}=judge_G.*A_G+ (~judge_G).*B_G;
        end
        Temp_M=M0A;% Or Temp_M=M0B;
        figure,plot_geometry(GF,Temp_M,options);title('Geometry of average of fused image F')
end
%%
disp('forward bandelet transform...')
switch Geometry_Rule
    case 'average'
        MBA = perform_bandlet_transform( M0A, G_ave, options );
        MBB = perform_bandlet_transform( M0B, G_ave, options );
    case 'maximum'
        MBA = perform_bandlet_transform( M0A, GA, options );
        MBB = perform_bandlet_transform( M0B, GB, options );
end
%===========================
L = length(MBA);
Fused=MBA;
%%
disp('fusion in high-frequency coefficients...')
for l = 1:L-1    
     Ahigh = MBA{l};
     Bhigh = MBB{l};
     judge_coef=abs(Ahigh)>=abs(Bhigh);
     Fused{l}=judge_coef.*Ahigh+ (~judge_coef).*Bhigh;
end
%%
disp('fusion in low-frequency coefficients...')
     Alow = MBA{L};
     Blow = MBB{L};
     %%%%%%%%%%%%%
     switch Low_Coeffs_Rule
         case 'average'
             Fused{L}=(Alow+Blow)/2;
         case 'maximum'
             judge_coef_low=abs(Alow)>=abs(Blow);
             Fused{L}=judge_coef.*Alow+ (~judge_coef).*Blow;
     end
%%
disp('inverse bandelet transform...')
M1= perform_bandlet_transform( Fused, GF, options );
max(M1(:))
min(M1(:))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
% figure,imshow(F,[])
 disp('F<0')
 sum(sum(M1<0))
 M1(M1<0)=0;
% figure,imshow(F,[])
 disp('F>255')
 sum(sum(M1>255))
 M1(M1>255)=255;
% figure,imshow(F,[])
F=round(M1);
%%
grey_level=256;
Criteria=Evaluation(M0A,M0B,F,grey_level)
%%
figure,imshow(M0A,[]),title('Souce image A');
figure,imshow(M0B,[]),title('Souce image B');
figure,imshow(F,[]),title('Fused image F');
figure,imshow(F-M0A,[]),title('Difference between A and F');
figure,imshow(F-M0B,[]),title('Difference between B and F');