function T = compute_best_threshold(M,M0,sigma,options, G)

% compute_best_threshold - oracle estimation of the threshold
%
% 	T = compute_best_threshold(M,M0,sigma,options, G);
%
%	M is the noisy image.
%	M0 is the original image.
%	sigma is the noise level.
%	G is the geometry (optional).
%
%	Test a range of threshold and find the best.
%	Leave G undefined for wavelet transform.
%	Set G to actual geometry for bandlet transform.
%
%	Copyright (c) 2007 Gabriel Peyre

if nargin<5
    G = []; 
end

niter = 8;
Tlist = linspace(2,3.5,niter)*sigma;

err = [];
for i = 1:niter
    T = Tlist(i);
    progressbar(i,niter);
    
    if isempty(G)
        % wavelets
        Jmin = 4;
        MW = perform_atrou_transform(M,Jmin,options);
        MWT = perform_thresholding(MW, T);
        M1 = perform_atrou_transform(MWT,Jmin,options);
    else
        % bandlets
        MB = perform_bandlet_transform( M, G, options );
        % perform thresholding
        MBT = perform_thresholding(MB, T);
        % reconstruct
        M1 = perform_bandlet_transform( MBT, G, options );
    end
    err(i) = psnr(M0,M1);
end

[tmp,I] = max(err);

if I==1 || I==niter
    warning('Out of bound reached.');
end
T = Tlist(I);