function plot_geometry(G, M, options)

% plot_quadtree - graphical display of a quatree
%
%   plot_geometry(G, M, options);
%
%   G is a matrix containing the chosen geometry inside each square.
%   M is an arbitrary background image
%
%   Copyright (c) 2005 Gabriel Peyr?


if nargin<2
    M = [];
end
if iscell(G)
    G = G{1};
end
if nargin<3
    nT = ceil(size(G,3)/2);
end

% width of the squares
n = size(M);
w = floor( n ./ size(G) );

options.null = 0;
if isfield(options, 'w')
    w = options.w;
else
    if isempty(M)
        error('You must provide options.w');
    end
end

if isfield(options, 'q')
    if not(w==options.q)
        error('Works only for w=q');
    end
end

if size(w,2)==1
    w = [w w];
end

nw = size(G);

str = 'r';
str_add = 'b';


hold on;
% display image with an overlaid grid
imagesc([1/2,n(1)-1/2], [1/2,n(2)-1/2], M);

[L,Theta] = compute_geometric_lut(w(1));

lw = 2;

% display subdivision
for kx=1:nw(1)
    for ky=1:nw(2)
        % center of the square
        pos = [kx-1,ky-1].*w + w/2;
        if G(kx,ky)==0
            % no geometry
        elseif G(kx,ky)<=length(Theta)
            % plot a geometry
            theta = Theta(G(kx,ky));
            r = min(w)/2;
            x = pos(1) + [cos(theta), -cos(theta)]*r;
            y = pos(2) + [sin(theta), -sin(theta)]*r;
            h = plot(y,x, str);
            set(h, 'LineWidth', lw);
        end
    end
end



% draw the lines
nmax = size(G).*w;
for kx = 0:w(1):n(1)
    x = [kx kx]; y = [0 nmax(2)];
    h = plot(y,x, 'k');
    set(h, 'LineWidth', lw);
end
for kx = 0:w(2):n(2)
    x = [kx kx]; y = [0 nmax(1)];
    h = plot(x,y, 'k');
    set(h, 'LineWidth', lw);
end

hold off;

axis tight;
axis square;
axis ij;
axis off;
colormap gray(256); 