% test for denoising using the bandlet transform
%
%   Copyright (c) 2007 Gabriel Peyre

path(path, 'toolbox/');
path(path,'F:\Matlab work\bandlet-denoising')
path(path,'F:\Matlab work\bandlet-denoising\toolbox\cwpt2')
path(path,'F:\Matlab work\bandlet-denoising\toolbox\mex')
path(path,'F:\Matlab work\bandlet-denoising\toolbox\rwt')
path(path,'F:\Matlab work\rice wavelet')
% name = 'peppers';
% name = 'boat';
% name = 'polygons_blurred';
% name = 'barb';
name = 'lena';
% name='sky_star'

n = 128;
M = load_image(name);
% M = load_image('square', n);
eta = 0.12;
c = size(M)/2;
if strcmp(name, 'lena') && n==128
    c = [120 200]; % lena hat
end
M0 = rescale( M(c(1)-n/2+1:c(1)+n/2,c(2)-n/2+1:c(2)+n/2 ), eta, 1-eta );

% add some noise
sigma = 0.04 * max( M0(:) );
randn('state',1234);    % to have reproductible results
M = M0 + randn(n)*sigma;

T = 3*sigma;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TI Wavelets
clear options;
options.wavelet_type = 'biorthogonal';
options.wavelet_vm = 3;
Jmin = 4;
options.decomp_type = 'quad';
MW = perform_atrou_transform(M,Jmin,options);
disp('--> Computing best threshold (wavelets).');
Twav = compute_best_threshold(M,M0,sigma, options);
MWT = perform_thresholding(MW, T);
MW1 = perform_atrou_transform(MWT,Jmin,options);

nbr_scales = 3; % number of scale on witch geometry is computed
bandlet_type = 'atrou-quad';
switch bandlet_type
    case 'atrou-quad'
        callback = @callback_atrou;
        options.decomp_type = 'quad';
        Jband = 3*nbr_scales;
    case 'atrou-tri'
        callback = @callback_atrou;
        options.decomp_type = 'tri';
        Jband = 2*nbr_scales;
    case 'steerable'
        options.nb_orientations = 1;
        callback = @callback_atrou;
        Jband = options.nb_orientations*nbr_scales+1;
    case 'laplacian'
        callback = @callback_laplacian;
        Jband = nbr_scales;
    otherwise
        error('Unknown type.');
end
        

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TI Bandlets
options.w = 8;
options.q = 8;  % for overlap set q=w/2, otherwise q=w
options.T = T;  % 10/256;
options.Jband = Jband; % number of scales for bandlets
options.Jhaar = 2; % number of scales for the 1D haar transform
options.callback = callback;
options.geometry_weight = 1; % price to pay for the geometry
options.n = n;

% use or not TI haar
options.ti = 1;

% compute the geometry for the bandlet transform
disp('--> Computing geometry.');
G = compute_geometry( M, options );
% forward transform
MB = perform_bandlet_transform( M, G, options );
% perform thresholding
disp('--> Computing best threshold (bandlets).');
Tband = compute_best_threshold(M,M0,sigma,options, G);
MBT = perform_thresholding(MB, Tband);
% reconstruct
MB1 = perform_bandlet_transform( MBT, G, options );

plot_geometry(G{1}, MW{1}, options);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Portilla and Simoncelli method
options.sigma = sigma;
options.repres2 = 'daub3';
options.parent = 1;
disp('--> BLS-GSM denoising.');
MP1 = perform_blsgsm_denoising(M, options);


pnoisy = psnr(M,M0);
pwav = psnr(M0,MW1);
pband = psnr(M0,MB1);
pbls= psnr(M0,MP1);

% plot_geometry(G{1}, MW{1}, options);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Display
display_image_layout({M0 M MW1 MB1 MP1}, ...
    {'Original' sprintf('Noisy,psnr=%.2f', pnoisy) sprintf('Wav,psnr=%.2f', pwav) ...
        sprintf('BLS,psnr=%.2f(+%.2f)', pbls, pbls-pwav) ...
        sprintf('Band,psnr=%.2f(+%.2f)', pband, pband-pwav)  });

% save
repimg = ['results/denoising/'];
if not(exist(repimg))
    mkdir(repimg);
end
saveas(gcf, [repimg name '-denoising.png'], 'png');

repimg = ['results/denoising/' name '/'];
if not(exist(repimg))
    mkdir(repimg);
end
warning off;
imwrite(clamp(M0), [repimg name '-original.png'], 'png');
imwrite(clamp(M), [repimg name '-noisy.png'], 'png');
imwrite(clamp(MW1), [repimg name '-wavelets.png'], 'png');
imwrite(clamp(MB1), [repimg name '-bandlets.png'], 'png');
warning off;